// 	UMouseTrackBehavior.h
// 	Copyright  1992 by Apple Computer, Inc. All rights reserved.
//	Kent Sandvik DTS
//	This file contains the basic TMouseTrackBehavior class definitions
//	Version Info (latest first):
//
//	<1>		khs		1.0		First final version
//	<2>		khs		1.0.1	Fixed a memory leak in TMapApplication::GetSleepValue()


#ifndef __MOUSETRACKBEHAVIOR__
#define __MOUSETRACKBEHAVIOR__


#ifndef __BEHAVIOR__
#include <UBehavior.h>
#endif

#ifndef __FLOATWINDOW__
#include <UFloatWindow.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCECONSTANTS__
#include "ResourceConstants.h"
#endif

// FORWARD STATEMENTS
class TMouseTrackBehavior;
class TSwallowBehavior;
class TTrackWindow;


// GLOBALS
const IDType kMouseTrackBehavior = 'MTRB';

TTrackWindow* gMouseTrackWindow = NULL;			// global TrackWindow hook (one and only)

pascal void InitMouseTrackBehavior();			// initialize our nice behavior (called once)


//	CLASSES
// floating window used for presentation of the mouse information
class TTrackWindow : public TFloatWindow
{
public:
	virtual pascal void Close();
	virtual pascal void GetOriginatorBehavior(TSwallowBehavior*);

	TSwallowBehavior* fBehavior;
};


//	our event swallow behavior, swallows everything
class TSwallowBehavior : public TBehavior
{
public:
	virtual pascal void ISwallowBehavior();
	virtual pascal Boolean DoToolboxEvent(TToolboxEvent* event);
	virtual pascal void Draw(const VRect& area);

	//	FIELDS
	TTrackWindow* fTrackWindow;
	TStaticText* fMouseStatus;
	TStaticText* fVertical;
	TStaticText* fHorizontal;
	VPoint fMouseVPoint;
	short fEventType;
};


//	the main behavior, which will coordinate swallow behavior and other objects
class TMouseTrackBehavior : public TBehavior
{
public:
	virtual pascal void IMouseTrackBehavior(ResNumber menuID);
	virtual pascal void Initialize();

	virtual pascal void DoSetupMenus();
	virtual pascal void DoMenuCommand(CommandNumber);
	virtual pascal void CreateSwallowBehavior();
	virtual pascal void CreateWindoid();

	//	FIELDS
	short fMenuID;
	TTrackWindow* fWindoid;
};




#endif __MOUSETRACKBEHAVIOR__

