/* 	TTracer 1.0 - example of a stack/heap based tracing class, which could be
	used for tracing memory leaks and keeping track of created objects
	See example below for use
	Copyright Apple, Inc. All Rights Reserved. Hah.
	Kent Sandvik/DTS - 1992.
	TODO: Create an association list for keeping track of what object is
 	active/inactive
*/

#include <stream.h>

#define TRACEPOINT __FILE__,__LINE__
// make use of the ANSI __FILE__ and __LINE__ macros

class TTracer {
public:
				TTracer(const char* className, const char* = 0, int = 0);
	virtual		~TTracer();
private:
	const char*	fLabel;
	const char* fFile;
	int			fLine;
	static int 	fReferenceCount;  // keep track of how many TTracers we construct
};

TTracer::TTracer(const char* label, const char* file, int line) :  
			fLabel(label), fFile(file), fLine(line)
{
	fReferenceCount++;
	cerr << "File " << fFile <<" ; Line " << fLine <<"  #+++ constructor event in "
			<< fLabel << " (reference count = " << fReferenceCount << ")\n";
}


TTracer::~TTracer()
{
	fReferenceCount--;
	cerr << "File " << fFile <<" ; Line " << fLine << "  #--- destructor  event in "
			<< fLabel << " (reference count = " << fReferenceCount << ")\n";
}

int TTracer::fReferenceCount = 0;					// initialize with 0 value

TTracer gGlobalTracer("gGlobalTracer", TRACEPOINT); 
// this will construct a global/universal tracer


// example of use:

void InvertPermutation(int* perm, int* inv, int max)
{
	TTracer autoTracer("InvertPermutation function", TRACEPOINT);
	
	if(perm && (new TTracer("temp", TRACEPOINT))   	// show TTracer in action
			&& inv
			&& (new TTracer("temp2", TRACEPOINT))  	// these two are never destructed =
			&& (max > 0))                          	// memory leak!
	{
		TTracer otherTracer("otherTracer", TRACEPOINT);
		
		for(int i = 0; i < max; i++){
			TTracer thirdTracer("iterationTracing...",TRACEPOINT);
			inv[perm[i]] = i;
		}
	}
}


// array declarations
int perm [] = {1, 2, 3, 6, 7};
int max = 5;


main()
{
	int* inv = new int[max];
	InvertPermutation(&perm[0], inv, max);
	
	return 0;
}
